#include <WiFiNINA.h>
#include <Servo.h>

// WiFi credentials
// For robotic Arm 2, mac address is 30:C6:F7:03:60:80
// the password is pUguZXWtBDzl
const char ssid[] = "SOTON-IoT";    // Your network SSID
const char pass[] = "pUguZXWtBDzl";    // Your network password
int status = WL_IDLE_STATUS;

// Server settings
WiFiServer server(80);

// Create servo objects
Servo index_f;
Servo middle;
Servo ring;
Servo pinky;

// Servo pins
const int indexPin = 5;
const int middlePin = 4;
const int ringPin = 3;
const int pinkyPin = 2;

void setup() {
  Serial.begin(9600);
  
  // Initialize servos
  index_f.attach(indexPin);
  middle.attach(middlePin);
  ring.attach(ringPin);
  pinky.attach(pinkyPin);
  
  // Set initial positions
  index_f.write(90);
  middle.write(90);
  ring.write(90);
  pinky.write(90);
  
  // Check for WiFi module
  if (WiFi.status() == WL_NO_MODULE) {
    Serial.println("Communication with WiFi module failed!");
    while (true);
  }

  // Attempt to connect to WiFi network
  while (status != WL_CONNECTED) {
    Serial.print("Attempting to connect to Network: ");
    Serial.println(ssid);
    status = WiFi.begin(ssid, pass);
    delay(10000);
  }

  Serial.println("Connected to wifi");
  printWiFiStatus();
  
  server.begin();
}

void loop() {
  WiFiClient client = server.available();
  
  if (client) {
    Serial.println("New client connected");
    String request = "";
    boolean currentLineIsBlank = true;
    unsigned long timeout = millis();
    
    while (client.connected() && millis() - timeout < 1000) {
      if (client.available()) {
        char c = client.read();
        request += c;
        
        // If you've gotten to the end of the line (received a newline
        // character) and the line is blank, the http request has ended,
        // so you can send a reply
        if (c == '\n' && currentLineIsBlank) {
          // Send a standard http response header
          client.println("HTTP/1.1 200 OK");
          client.println("Content-Type: text/html");
          client.println("Connection: close");
          client.println();
          
          // Send a simple confirmation page
          client.println("<html><body>");
          client.println("<h1>Command received</h1>");
          client.println("</body></html>");
          break;
        }
        if (c == '\n') {
          currentLineIsBlank = true;
        } else if (c != '\r') {
          currentLineIsBlank = false;
        }
      }
    }
    
    // Process the request
    if (request.indexOf("GET /") != -1) {
      Serial.println("Received request: " + request);
      processRequest(request);
    }
    
    // Give the web browser time to receive the data
    delay(10);
    client.stop();
    Serial.println("Client disconnected");
  }
}

void processRequest(String request) {
  // Extract the command path
  int start = request.indexOf("GET ") + 4;
  int end = request.indexOf(" HTTP");
  
  if (start != -1 && end != -1) {
    String command = request.substring(start, end);
    Serial.println("Command: " + command);
    
    // Parse command and move servos
    if (command.startsWith("/index/")) {
      int angle = command.substring(7).toInt();
      angle = constrain(angle, 0, 90);
      index_f.write(angle);
      Serial.println("Moving index to " + String(angle));
    }
    else if (command.startsWith("/middle/")) {
      int angle = command.substring(8).toInt();
      angle = constrain(angle, 0, 90);
      middle.write(angle);
      Serial.println("Moving middle to " + String(angle));
    }
    else if (command.startsWith("/ring/")) {
      int angle = command.substring(6).toInt();
      angle = constrain(angle, 0, 90);
      ring.write(angle);
      Serial.println("Moving ring to " + String(angle));
    }
    else if (command.startsWith("/pinky/")) {
      int angle = command.substring(7).toInt();
      angle = constrain(angle, 0, 90);
      pinky.write(angle);
      Serial.println("Moving pinky to " + String(angle));
    }
  }
}

void printWiFiStatus() {
  Serial.print("SSID: ");
  Serial.println(WiFi.SSID());

  IPAddress ip = WiFi.localIP();
  Serial.print("IP Address: ");
  Serial.println(ip);

  long rssi = WiFi.RSSI();
  Serial.print("Signal strength (RSSI):");
  Serial.print(rssi);
  Serial.println(" dBm");
}